//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/EnvironmentEditor.cpp
//! @brief     Implements class EnvironmentEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/EnvironmentEditor.h"
#include "GUI/Model/Device/BackgroundItems.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/View/Common/DoubleSpinBox.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"
#include "GUI/View/Tool/WidgetUtils.h"
#include <QFormLayout>

EnvironmentEditor::EnvironmentEditor(QWidget* parent, InstrumentItem* instrument)
    : QGroupBox("Environment", parent)
    , m_instrument(instrument)
{
    ASSERT(instrument);
    m_formLayout = new QFormLayout(this);
    m_formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);

    auto* backgroundTypeCombo =
        GUI::Util::createComboBoxFromDescriptor(instrument->background().m_descriptor, [=](int) {
            createBackgroundWidgets();
            emit dataChanged();
        });
    m_formLayout->addRow("Background type:", backgroundTypeCombo);

    createBackgroundWidgets();
    GroupBoxCollapser::installIntoGroupBox(this);
}

void EnvironmentEditor::createBackgroundWidgets()
{
    while (m_formLayout->rowCount() > 1)
        m_formLayout->removeRow(1);

    auto* backgroundItem = m_instrument->background().currentItem();
    if (auto* p = dynamic_cast<ConstantBackgroundItem*>(backgroundItem)) {
        auto* spinbox = new DoubleSpinBox(p->backgroundValue());
        spinbox->setSingleStep(0.01);
        m_formLayout->addRow("Background value:", spinbox);

        connect(spinbox, &DoubleSpinBox::baseValueChanged, [=](double newValue) {
            p->setBackgroundValue(newValue);
            emit dataChanged();
        });
    }
}
