//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/ResolutionFunctionEditor.cpp
//! @brief     Implements class ResolutionFunctionEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/ResolutionFunctionEditor.h"
#include "GUI/Model/Device/DetectorItems.h"
#include "GUI/Model/Device/ResolutionFunctionItems.h"
#include "GUI/View/Common/DoubleSpinBox.h"
#include "GUI/View/Tool/GroupBoxCollapser.h"
#include "GUI/View/Tool/WidgetUtils.h"
#include <QFormLayout>
#include <QGroupBox>

ResolutionFunctionEditor::ResolutionFunctionEditor(Unit unit, QWidget* parent, DetectorItem* item)
    : QGroupBox("Resolution function", parent)
    , m_unit(unit)
    , m_item(item)
{
    ASSERT(item);
    setProperty("subgroup", true); // for stylesheet addressing
    m_formLayout = new QFormLayout(this);
    m_formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);

    auto* typeCombo =
        GUI::Util::createComboBoxFromDescriptor(item->resolutionFunction().m_descriptor, [=](int) {
            createResolutionWidgets();
            emit dataChanged();
        });
    m_formLayout->addRow("Type:", typeCombo);

    GroupBoxCollapser::installIntoGroupBox(this);

    createResolutionWidgets();
}

void ResolutionFunctionEditor::createResolutionWidgets()
{
    while (m_formLayout->rowCount() > 1)
        m_formLayout->removeRow(1);

    auto* resFunction = m_item->resolutionFunction().currentItem();
    if (auto* p = dynamic_cast<ResolutionFunction2DGaussianItem*>(resFunction)) {
        auto* sigmaXSpinBox = GUI::Util::createDoubleSpinBoxRow(m_formLayout, p->sigmaX());
        auto* sigmaYSpinBox = GUI::Util::createDoubleSpinBoxRow(m_formLayout, p->sigmaY());

        connect(sigmaXSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
                [=](double newValue) {
                    p->setSigmaX(newValue);
                    emit dataChanged();
                });

        connect(sigmaYSpinBox, qOverload<double>(&DoubleSpinBox::baseValueChanged),
                [=](double newValue) {
                    p->setSigmaY(newValue);
                    emit dataChanged();
                });
    }
}
